import faker from 'faker';
import _ from 'lodash';

const authors = [];
const posts = [];
const comments = [];

for(let i = 0; i < 10; i++) {
  authors.push({
    __type: 'author',
    id: faker.random.uuid(),
    name: faker.name.findName(),
    email: faker.internet.email()
  });
}

for(let i = 0; i < 15; i++) {
  posts.push({
    __type: 'post',
    id: faker.random.uuid(),
    title: faker.lorem.sentence(),
    content: faker.lorem.paragraph(),
    author: faker.helpers.randomize(authors).id,
    date: faker.date.past()
  });
}

for(let i = 0; i < 50; i++) {
  comments.push({
    __type: 'comment',
    id: faker.random.uuid(),
    content: faker.lorem.sentence(),
    post: faker.helpers.randomize(posts).id,
    author: faker.helpers.randomize(authors).id,
    date: faker.date.past()
  });
}

const db = {
  posts,
  comments,
  authors,

  get: (type, id) => _.find(db[type + 's'], { id }),

  findAuthors: where => Promise.resolve(_.filter(authors, where)),

  contentByAuthor: authorID => _.shuffle(_.merge(
    _.filter(posts, { author: authorID }),
    _.filter(comments, { author: authorID })
  )),

  postsByAuthor: authorID => _.filter(posts, { author: authorID }),
  postComments: postID => _.filter(comments, { post: postID }),
  getAuthorById: authorID => _.find(authors, { id: authorID })
};

export default db;
