import Titanium from '../../src';
import db from './db';

// Make a new instance of Titanium. The Titanium class is responsible for
// holding the whole schema, with its type definitions, rootQuery fields and
// mutations.
const titanium = new Titanium({
  rootQuery: {
    // The rootQuery readable name and description (shows up on GraphiQL)
    name: 'BlogRoot',
    description: `A cool and neat blog schema for cool developers!
    Remember, you can even use **Markdown** syntax!`
  },

  // this is a relay convention specific thing: we implement by default a node
  // field inside the rootQuery in order to query any node on the database by
  // its unique ID.
  nodeResolver: (type, id, { tql }) => {
    const node = db.get(type, id);

    if(!node) {
      tql.throw(404, `A node with the provided ID was not found on database.`);
    }

    return db.get(type, id);
  }
});

titanium.addStoreStrategy({
  name: 'author',
  resolve: authors =>
    db.findAuthors(author => authors.indexOf(author.id) !== -1)
});

export default titanium;
