import titanium from '../root';
import db from '../db';
import { TitaniumObjectType } from '../../../src';

const postType = new TitaniumObjectType({
  name: 'Post',
  description: 'A post on a blog',
  isTypeOf: node => (node.__type === 'post'),
  fields: {
    id: 'id',
    title: 'string',
    content: 'string',
    date: 'datetime',
    author: {
      type: 'reference',
      ofType: 'author',
      // look! DataLoader out of the box :)
      resolve: (node, args, { tql }) => tql.stores.author.load(node.author)
    },
    comments: {
      type: 'connection',
      ofType: 'comment',
      resolve: node => db.postComments(node.id)
    }
  },
  interfaces: [ 'Node' ],
  resolveAll: () => db.posts
});

postType.setOutputRules({
  title: {
    read: false
  }
});

// we apply the new type definition to the Titanium main schema
titanium.addType(postType);
