/* eslint camelcase: 0 */

/* author notice: we disable eslint 'camelcase' config here due to
 * the nature of graphql fields being declared with
 * snake_case by default. */

import Koa from 'koa';
import mount from 'koa-mount';
import graphqlHTTP from 'koa-graphql';
import Titanium, {
  TitaniumObjectType,
  TitaniumQuery
} from '../../src';

// Make a new instance of Titanium. The Titanium class is responsible for
// holding the whole schema, with its type definitions, rootQuery fields and
// mutations.
const titanium = new Titanium({
  rootQuery: {
    // The rootQuery readable name and description (shows up on GraphiQL)
    name: 'TQLRoot',
    description: `Your neat and nice rootQuery`
  },

  // this is a relay convention specific thing: we implement by default a node
  // field inside the rootQuery in order to query any node on the database by
  // its unique ID.
  nodeResolver: node => node
});

const versionType = new TitaniumObjectType({
  name: 'TitaniumVersion',
  createHelpers: false,
  description: 'Version details of current running Titanium',
  fields: {
    semver: 'string',
    release_date: 'datetime',
    author: 'string'
  }
});

// we apply the new type definition to the Titanium main schema
titanium.addType(versionType);

const versionQuery = new TitaniumQuery({
  name: 'version',
  type: 'reference',
  ofType: 'titaniumversion',
  resolve: () => ({
    semver: '0.0.1',
    release_date: Date.now(),
    author: 'Vinicius Tavares'
  })
});

// we apply the new query to the Titanium schema (will be a direct child of
// the rootQuery)
titanium.addQuery(versionQuery);

// resolve the schema and convert it to GraphQL
const schema = titanium.getSchema();

const app = Koa();

app.use(mount('/', graphqlHTTP({
  schema,
  graphiql: true,
  formatError(error) {
    console.error(error.stack);

    return {
      message: error.message,
      locations: error.locations
    };
  }
})));

const examplePort = process.env.PORT || 3000;
app.listen(examplePort, () => {
  console.log(
    `Ready to rock! ` +
    `Open http://localhost:${examplePort}/ at your browser.`
  );
});
