'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _vm = require('vm2');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var TitaniumRules = function () {
  function TitaniumRules(fieldsList) {
    _classCallCheck(this, TitaniumRules);

    if (!(fieldsList instanceof Object)) {
      throw new TypeError('Value of argument "fieldsList" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(fieldsList));
    }

    this._fieldRules = fieldsList;
  }

  _createClass(TitaniumRules, [{
    key: '_getSandbox',
    value: function _getSandbox(node, context, fieldName) {
      if (!(node == null || node instanceof Object)) {
        throw new TypeError('Value of argument "node" violates contract.\n\nExpected:\n?Object\n\nGot:\n' + _inspect(node));
      }

      if (!(context instanceof Object)) {
        throw new TypeError('Value of argument "context" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(context));
      }

      if (!(fieldName == null || typeof fieldName === 'string')) {
        throw new TypeError('Value of argument "fieldName" violates contract.\n\nExpected:\n?string\n\nGot:\n' + _inspect(fieldName));
      }

      var sandbox = {
        $user: context.user || {},
        $node: node,
        $newData: {},
        now: Date.now()
      };

      sandbox.$user.isLoggedIn = function () {
        return typeof user !== 'undefined';
      };

      if (fieldName) {
        sandbox.$newData.type = function () {
          return _typeof(node[fieldName]);
        };

        sandbox.$newData.val = function () {
          return node[fieldName];
        };
      }

      return sandbox;
    }
  }, {
    key: '_runVM',
    value: function _runVM(sandbox, script) {
      if (!(sandbox instanceof Object)) {
        throw new TypeError('Value of argument "sandbox" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(sandbox));
      }

      if (!(typeof script === 'string')) {
        throw new TypeError('Value of argument "script" violates contract.\n\nExpected:\nstring\n\nGot:\n' + _inspect(script));
      }

      var vm = new _vm.VM({ sandbox: sandbox, timeout: 1000 });
      return vm.run(script);
    }
  }, {
    key: '_parseRule',
    value: function _parseRule(node, context, fieldName, verb) {
      if (!(typeof fieldName === 'string')) {
        throw new TypeError('Value of argument "fieldName" violates contract.\n\nExpected:\nstring\n\nGot:\n' + _inspect(fieldName));
      }

      if (!(typeof verb === 'string')) {
        throw new TypeError('Value of argument "verb" violates contract.\n\nExpected:\nstring\n\nGot:\n' + _inspect(verb));
      }

      if (typeof this._fieldRules[fieldName] === 'undefined' || typeof this._fieldRules[fieldName][verb] === 'undefined') {
        if (verb === 'sanitize') {
          // defaults to current value
          return node[fieldName];
        }

        // defaults to true
        return true;
      }

      var rule = this._fieldRules[fieldName][verb];

      if (typeof rule === 'boolean') {
        return rule;
      }

      if (typeof rule === 'string') {
        var sandbox = this._getSandbox(node, context, fieldName);
        return this._runVM(sandbox, rule);
      }

      if (typeof rule === 'function') {
        return rule(node, context, fieldName);
      }

      if (verb === 'validate' && rule instanceof RegExp) {
        return rule.test(node[fieldName]);
      }

      throw new Error('Sorry, we don\'t know how to deal with rules of type ' + ('\'' + (typeof rule === 'undefined' ? 'undefined' : _typeof(rule)) + '\' for field \'' + fieldName + '\''));
    }
  }, {
    key: 'affects',
    value: function affects(fieldName) {
      function _ref4(_id4) {
        if (!(typeof _id4 === 'boolean')) {
          throw new TypeError('Function return value violates contract.\n\nExpected:\nbool\n\nGot:\n' + _inspect(_id4));
        }

        return _id4;
      }

      if (!(typeof fieldName === 'string')) {
        throw new TypeError('Value of argument "fieldName" violates contract.\n\nExpected:\nstring\n\nGot:\n' + _inspect(fieldName));
      }

      return _ref4(typeof this._fieldRules[fieldName] !== 'undefined');
    }
  }, {
    key: 'validate',
    value: function validate(node, context, fieldName) {
      function _ref5(_id5) {
        if (!(typeof _id5 === 'boolean')) {
          throw new TypeError('Function return value violates contract.\n\nExpected:\nbool\n\nGot:\n' + _inspect(_id5));
        }

        return _id5;
      }

      if (!(node instanceof Object)) {
        throw new TypeError('Value of argument "node" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(node));
      }

      if (!(context instanceof Object)) {
        throw new TypeError('Value of argument "context" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(context));
      }

      if (!(typeof fieldName === 'string')) {
        throw new TypeError('Value of argument "fieldName" violates contract.\n\nExpected:\nstring\n\nGot:\n' + _inspect(fieldName));
      }

      return _ref5(this._parseRule(node, context, fieldName, 'validate'));
    }
  }, {
    key: 'sanitize',
    value: function sanitize(node, context, fieldName) {
      if (!(node instanceof Object)) {
        throw new TypeError('Value of argument "node" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(node));
      }

      if (!(context instanceof Object)) {
        throw new TypeError('Value of argument "context" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(context));
      }

      if (!(typeof fieldName === 'string')) {
        throw new TypeError('Value of argument "fieldName" violates contract.\n\nExpected:\nstring\n\nGot:\n' + _inspect(fieldName));
      }

      return this._parseRule(node, context, fieldName, 'sanitize');
    }
  }, {
    key: 'isReadable',
    value: function isReadable(node, context, fieldName) {
      function _ref7(_id7) {
        if (!(typeof _id7 === 'boolean')) {
          throw new TypeError('Function return value violates contract.\n\nExpected:\nbool\n\nGot:\n' + _inspect(_id7));
        }

        return _id7;
      }

      if (!(node instanceof Object)) {
        throw new TypeError('Value of argument "node" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(node));
      }

      if (!(context instanceof Object)) {
        throw new TypeError('Value of argument "context" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(context));
      }

      if (!(typeof fieldName === 'string')) {
        throw new TypeError('Value of argument "fieldName" violates contract.\n\nExpected:\nstring\n\nGot:\n' + _inspect(fieldName));
      }

      return _ref7(this._parseRule(node, context, fieldName, 'read'));
    }
  }, {
    key: 'isWritable',
    value: function isWritable(node, context, fieldName) {
      function _ref8(_id8) {
        if (!(typeof _id8 === 'boolean')) {
          throw new TypeError('Function return value violates contract.\n\nExpected:\nbool\n\nGot:\n' + _inspect(_id8));
        }

        return _id8;
      }

      if (!(node instanceof Object)) {
        throw new TypeError('Value of argument "node" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(node));
      }

      if (!(context instanceof Object)) {
        throw new TypeError('Value of argument "context" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(context));
      }

      if (!(typeof fieldName === 'string')) {
        throw new TypeError('Value of argument "fieldName" violates contract.\n\nExpected:\nstring\n\nGot:\n' + _inspect(fieldName));
      }

      return _ref8(this._parseRule(node, context, fieldName, 'write'));
    }
  }]);

  return TitaniumRules;
}();

exports.default = TitaniumRules;

function _inspect(input, depth) {
  var maxDepth = 4;
  var maxKeys = 15;

  if (depth === undefined) {
    depth = 0;
  }

  depth += 1;

  if (input === null) {
    return 'null';
  } else if (input === undefined) {
    return 'void';
  } else if (typeof input === 'string' || typeof input === 'number' || typeof input === 'boolean') {
    return typeof input === 'undefined' ? 'undefined' : _typeof(input);
  } else if (Array.isArray(input)) {
    if (input.length > 0) {
      var _ret = function () {
        if (depth > maxDepth) return {
            v: '[...]'
          };

        var first = _inspect(input[0], depth);

        if (input.every(function (item) {
          return _inspect(item, depth) === first;
        })) {
          return {
            v: first.trim() + '[]'
          };
        } else {
          return {
            v: '[' + input.slice(0, maxKeys).map(function (item) {
              return _inspect(item, depth);
            }).join(', ') + (input.length >= maxKeys ? ', ...' : '') + ']'
          };
        }
      }();

      if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
    } else {
      return 'Array';
    }
  } else {
    var keys = Object.keys(input);

    if (!keys.length) {
      if (input.constructor && input.constructor.name && input.constructor.name !== 'Object') {
        return input.constructor.name;
      } else {
        return 'Object';
      }
    }

    if (depth > maxDepth) return '{...}';
    var indent = '  '.repeat(depth - 1);
    var entries = keys.slice(0, maxKeys).map(function (key) {
      return (/^([A-Z_$][A-Z0-9_$]*)$/i.test(key) ? key : JSON.stringify(key)) + ': ' + _inspect(input[key], depth) + ';';
    }).join('\n  ' + indent);

    if (keys.length >= maxKeys) {
      entries += '\n  ' + indent + '...';
    }

    if (input.constructor && input.constructor.name && input.constructor.name !== 'Object') {
      return input.constructor.name + ' {\n  ' + indent + entries + '\n' + indent + '}';
    } else {
      return '{\n  ' + indent + entries + '\n' + indent + '}';
    }
  }
}