'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.TitaniumFieldMapper = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _graphqlRelay = require('graphql-relay');

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _type = require('./type');

var _resolve = require('./resolve');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

// the main fieldMapper. takes the field and manipulates it accordingly.
// maps name and description, gets deprecation data and so on.

var TitaniumFieldMapper = exports.TitaniumFieldMapper = function () {
  function TitaniumFieldMapper(options) {
    _classCallCheck(this, TitaniumFieldMapper);

    var typeName = options.typeName;
    var typeFields = options.typeFields;
    var typeRules = options.typeRules;


    this._typeName = typeName;
    // @TODO: add invariant for field checking
    this._typeFields = typeFields;
    this._typeRules = typeRules;
  }

  _createClass(TitaniumFieldMapper, [{
    key: '_toGraphQL',
    value: function _toGraphQL(typeDefinitionList) {
      var _this = this;

      function _ref(_id) {
        if (!(_id instanceof Object)) {
          throw new TypeError('Function return value violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(_id));
        }

        return _id;
      }

      if (!(typeDefinitionList instanceof Object)) {
        throw new TypeError('Value of argument "typeDefinitionList" violates contract.\n\nExpected:\nObject\n\nGot:\n' + _inspect(typeDefinitionList));
      }

      (0, _invariant2.default)((typeof typeDefinitionList === 'undefined' ? 'undefined' : _typeof(typeDefinitionList)) === 'object' && !Array.isArray(typeDefinitionList), 'fieldMapper expects \'typeDefinitionList\' to be of type \'Object\', ' + ('received \'' + (typeof typeDefinitionList === 'undefined' ? 'undefined' : _typeof(typeDefinitionList)) + '\' instead'));

      this._typeDefinitionList = typeDefinitionList;

      return _ref(_lodash2.default.mapValues(this._typeFields, function (fieldDefinition, fieldName) {
        (0, _invariant2.default)(typeof fieldName === 'string', 'fieldMapper expects \'fieldName\' to be of type \'String\', ' + ('received \'' + (typeof fieldName === 'undefined' ? 'undefined' : _typeof(fieldName)) + '\' instead'));

        (0, _invariant2.default)((typeof fieldDefinition === 'undefined' ? 'undefined' : _typeof(fieldDefinition)) !== undefined, 'fieldMapper requires field \'fieldDefinition\'');

        var returnObject = {
          type: (0, _type.convertType)({
            typeName: _this._typeName,
            typeDefinitionList: _this._typeDefinitionList,
            fieldName: fieldName,
            fieldDefinition: fieldDefinition
          })
        };

        var fieldResolver = (0, _resolve.convertResolve)({
          fieldName: fieldName,
          fieldDefinition: fieldDefinition,
          typeRules: _this._typeRules
        });

        if (returnObject.type === 'id') {
          returnObject = (0, _graphqlRelay.globalIdField)(_this._typeName.toLowerCase(), fieldResolver);
        } else {
          returnObject.name = fieldDefinition.name || fieldName;
          returnObject.description = fieldDefinition.description || '';
          returnObject.resolve = fieldResolver;
        }

        if (typeof fieldDefinition.args !== 'undefined') {
          returnObject.args = _lodash2.default.mapValues(fieldDefinition.args, function (argDefinition, argName) {
            return {
              type: (0, _type.convertType)({
                typeName: _this._typeName,
                typeDefinitionList: _this._typeDefinitionList,
                fieldName: argName,
                fieldDefinition: argDefinition,
                ignoreIDFieldName: true
              })
            };
          });
        }

        if (fieldDefinition.type === 'connection') {
          returnObject.args = returnObject.args || {};

          var intersection = _lodash2.default.intersection(Object.keys(returnObject.args), Object.keys(_graphqlRelay.connectionArgs));

          (0, _invariant2.default)(intersection.length === 0, 'Warning: Some defined args for field ' + fieldName + ' will be ' + 'overwritten by connection args!');

          _lodash2.default.assign(returnObject.args, _graphqlRelay.connectionArgs);
        }

        if (fieldDefinition.isDeprecated) {
          returnObject.isDeprecated = true;

          returnObject.deprecationReason = fieldDefinition.deprecationReason || null;
        }

        return returnObject;
      }));
    }
  }]);

  return TitaniumFieldMapper;
}();

function _inspect(input, depth) {
  var maxDepth = 4;
  var maxKeys = 15;

  if (depth === undefined) {
    depth = 0;
  }

  depth += 1;

  if (input === null) {
    return 'null';
  } else if (input === undefined) {
    return 'void';
  } else if (typeof input === 'string' || typeof input === 'number' || typeof input === 'boolean') {
    return typeof input === 'undefined' ? 'undefined' : _typeof(input);
  } else if (Array.isArray(input)) {
    if (input.length > 0) {
      var _ret = function () {
        if (depth > maxDepth) return {
            v: '[...]'
          };

        var first = _inspect(input[0], depth);

        if (input.every(function (item) {
          return _inspect(item, depth) === first;
        })) {
          return {
            v: first.trim() + '[]'
          };
        } else {
          return {
            v: '[' + input.slice(0, maxKeys).map(function (item) {
              return _inspect(item, depth);
            }).join(', ') + (input.length >= maxKeys ? ', ...' : '') + ']'
          };
        }
      }();

      if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
    } else {
      return 'Array';
    }
  } else {
    var keys = Object.keys(input);

    if (!keys.length) {
      if (input.constructor && input.constructor.name && input.constructor.name !== 'Object') {
        return input.constructor.name;
      } else {
        return 'Object';
      }
    }

    if (depth > maxDepth) return '{...}';
    var indent = '  '.repeat(depth - 1);
    var entries = keys.slice(0, maxKeys).map(function (key) {
      return (/^([A-Z_$][A-Z0-9_$]*)$/i.test(key) ? key : JSON.stringify(key)) + ': ' + _inspect(input[key], depth) + ';';
    }).join('\n  ' + indent);

    if (keys.length >= maxKeys) {
      entries += '\n  ' + indent + '...';
    }

    if (input.constructor && input.constructor.name && input.constructor.name !== 'Object') {
      return input.constructor.name + ' {\n  ' + indent + entries + '\n' + indent + '}';
    } else {
      return '{\n  ' + indent + entries + '\n' + indent + '}';
    }
  }
}