'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.convertResolve = undefined;

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _graphqlRelay = require('graphql-relay');

var _type = require('./type');

var _utilities = require('../utilities');

var _acl = require('../acl');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// the resolver converter aims to give all fields a resolver.
// also, we have some built-in resolvers as datetime, so the user just
// needs to set the field type as datetime and the correct resolver will
// be applied. if no resolver is supplied, the default one will be applied.
var _convert = function _convert(options) {
  var typeName = options.typeName;
  var flatType = options.flatType;
  var fieldName = options.fieldName;
  var fieldResolve = options.fieldResolve;

  // general

  if (fieldResolve && flatType !== 'connection') {
    // tries to find a resolver in the field, if not found, use the
    // regular one.
    return fieldResolve;
  }

  // virtual
  if (flatType === 'virtual') {
    (0, _invariant2.default)(typeof fieldResolve === 'function', 'The virtual field \'' + fieldName + '\' at type \'' + typeName + '\' must ' + 'have a valid resolver');

    return fieldResolve;
  }

  if (flatType === 'connection') {
    (0, _invariant2.default)(typeof fieldResolve === 'function', 'The connection field \'' + fieldName + '\' at type \'' + typeName + '\' must ' + 'have a valid resolver');

    // this code wraps the relay connection helper
    // to the resolver defined by the user
    return function (node, args, context, info) {
      return (0, _graphqlRelay.connectionFromPromisedArray)(
      // user is not enforced to return a Promise anymore
      Promise.resolve(fieldResolve(node, args, context, info)), args);
    };
  }

  return (0, _utilities.TitaniumResolver)(flatType) || (0, _utilities.TitaniumResolver)('regular');
};

var convertResolve = exports.convertResolve = function convertResolve(options) {
  var fieldName = options.fieldName;
  var fieldDefinition = options.fieldDefinition;
  var typeRules = options.typeRules;


  var flatType = (0, _type.flattenType)(fieldDefinition);

  var resolve = _convert({
    flatType: flatType,
    fieldName: fieldName,
    fieldResolve: fieldDefinition.resolve
  });

  if (typeRules && typeRules.affects(fieldName)) {
    return (0, _acl.wrapFieldRules)({
      resolve: resolve,
      fieldName: fieldName,
      typeRules: typeRules
    });
  }

  return resolve;
};