'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.convertType = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _graphql = require('graphql');

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var _normalizer = require('./normalizer');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var wrapRequiredField = function wrapRequiredField(fieldDefinition, type) {
  if (fieldDefinition.required && type !== 'id') {
    return new _graphql.GraphQLNonNull(type);
  }

  return type;
};

var maybeSimple = function maybeSimple(flatType) {
  if (flatType === 'boolean') {
    return _graphql.GraphQLBoolean;
  }

  if (flatType === 'float') {
    return _graphql.GraphQLFloat;
  }

  if (flatType === 'int') {
    return _graphql.GraphQLInt;
  }

  if (flatType === 'datetime' || flatType === 'dateonly' || flatType === 'time' || flatType === 'string') {
    return _graphql.GraphQLString;
  }

  return null;
};

var _typeConverter = function _typeConverter(typeConfig) {
  var flatType = typeConfig.flatType;
  var typeName = typeConfig.typeName;
  var fieldName = typeConfig.fieldName;
  var fieldDefinition = typeConfig.fieldDefinition;
  var typeDefinitionList = typeConfig.typeDefinitionList;
  var _typeConfig$ignoreIDF = typeConfig.ignoreIDFieldName;
  var ignoreIDFieldName = _typeConfig$ignoreIDF === undefined ? false : _typeConfig$ignoreIDF;

  // if the type is a common one, just return it already!

  var simpleType = maybeSimple(flatType);

  if (simpleType) {
    return simpleType;
  }

  if (flatType === 'array') {
    var _mapArrayType = (0, _normalizer.mapArrayType)(fieldDefinition);

    var subType = _mapArrayType.subType;
    var subValue = _mapArrayType.subValue;


    (0, _invariant2.default)(subType !== 'reference', 'You are trying to make a list of references at field \'' + fieldName + '\' ' + ('at type \'' + typeName + '\' using Arrays. Please use connection instead to ') + 'keep Relay\'s pattern.');

    (0, _invariant2.default)(subType !== 'array' && subType !== 'enum' && subType !== 'connection', 'You can not nest Array or Enum types at field \'' + fieldName + '\' of ' + ('type \'' + typeName + '\''));

    return new _graphql.GraphQLList(convertType({
      flatType: flatType,
      typeName: typeName,
      fieldName: fieldName,
      fieldDefinition: subValue,
      typeDefinitionList: typeDefinitionList
    }));
  }

  if (flatType === 'enum') {
    var _ret = function () {
      // Regex for finding special characters
      var specialChars = /[^a-z\d_]/i;

      var enumValues = fieldDefinition.enum.values.reduce(function (obj, value) {
        var sanitizedValue = value;
        if (specialChars.test(value)) {
          sanitizedValue = value.split(specialChars).reduce(function (reduced, val, idx) {
            var newVal = val;
            if (idx > 0) {
              newVal = '' + val[0].toUpperCase() + val.slice(1);
            }
            return '' + reduced + newVal;
          });
        }

        obj[sanitizedValue] = {
          value: value
        };

        return obj;
      }, {});

      return {
        v: new _graphql.GraphQLEnumType({
          name: fieldDefinition.enum.name || fieldDefinition.name || fieldName,
          description: fieldDefinition.enum.description || fieldDefinition.description || '',
          values: enumValues
        })
      };
    }();

    if ((typeof _ret === 'undefined' ? 'undefined' : _typeof(_ret)) === "object") return _ret.v;
  }

  if (flatType === 'virtual') {
    if (fieldDefinition.ofType) {
      return convertType({
        flatType: flatType,
        typeName: typeName,
        fieldName: fieldName,
        fieldDefinition: fieldDefinition.ofType,
        typeDefinitionList: typeDefinitionList
      });
    }

    return _graphql.GraphQLString;
  }

  if (flatType === 'reference') {
    (0, _invariant2.default)(typeof fieldDefinition.ofType !== 'undefined', 'Reference field \'' + fieldName + '\' must have its reference type ' + 'defined at \'ofType\' field');

    return typeDefinitionList[fieldDefinition.ofType];
  }

  if (flatType === 'connection') {
    (0, _invariant2.default)(typeof fieldDefinition.ofType !== 'undefined', 'Connection field \'' + fieldName + '\' must have its connection type ' + 'defined at \'ofType\' field');

    return typeDefinitionList[fieldDefinition.ofType + 'Connection'];
  }

  if (flatType === 'id') {
    if (!ignoreIDFieldName) {
      (0, _invariant2.default)(fieldName === 'id', 'Fields of type ID must be named as \'id\' in order to keep server\'s ' + 'consistency. However, if you need to map this field to another one ' + 'on your database, please use the idMapsTo config into your Titanium ' + 'instance instead.');
    }

    return 'id';
  }
};

// the master converter. receives the master fields reference
// object, the current field key and value returns the graphQL
// type corresponding to the field
var convertType = exports.convertType = function convertType(typeConfig) {
  var typeName = typeConfig.typeName;
  var fieldName = typeConfig.fieldName;
  var fieldDefinition = typeConfig.fieldDefinition;
  var typeDefinitionList = typeConfig.typeDefinitionList;
  var _typeConfig$ignoreIDF2 = typeConfig.ignoreIDFieldName;
  var ignoreIDFieldName = _typeConfig$ignoreIDF2 === undefined ? false : _typeConfig$ignoreIDF2;


  var flatType = (0, _normalizer.flattenType)(fieldDefinition);
  (0, _invariant2.default)(flatType !== null, 'Invalid type for field \'' + fieldName + '\'');

  return wrapRequiredField(fieldDefinition, _typeConverter({
    flatType: flatType,
    typeName: typeName,
    fieldName: fieldName,
    fieldDefinition: fieldDefinition,
    ignoreIDFieldName: ignoreIDFieldName,
    typeDefinitionList: typeDefinitionList
  }));
};