'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.mapArrayType = exports.flattenType = exports.arrayReference = exports.getTypeReference = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// sometimes a field can be declared only by its type
// for example: first_name: 'string'
// or complex types with name, dedicated resolve functions may appear such:
// best_friend: { type: 'connection', ofType: 'user' }
// for those cases we need to check which mode this field was declared.
var getTypeReference = exports.getTypeReference = function getTypeReference(fieldDefinition) {
  if ((typeof fieldDefinition === 'undefined' ? 'undefined' : _typeof(fieldDefinition)) === 'object' && !(fieldDefinition instanceof Array) && !fieldDefinition.isTitaniumObjectType) {

    if (typeof fieldDefinition.type === 'undefined') {
      return null;
    }

    return fieldDefinition.type;
  }

  return fieldDefinition;
};

// due to shorthand declarations, we need to check if the field was declared as:
// field: [type] or field { type: [type] }
// this function gets the correct var reference.
var arrayReference = exports.arrayReference = function arrayReference(fieldDefinition) {
  if (Array.isArray(fieldDefinition)) {
    return fieldDefinition;
  }

  return fieldDefinition.type;
};

// sometimes you want to use an array definition as ['string']
// this function normalizes the type declaration
var flattenType = exports.flattenType = function flattenType(fieldDefinition) {
  var typeReference = getTypeReference(fieldDefinition);

  if (typeof typeReference === 'string') {
    return typeReference.toLowerCase();
  }

  if (Array.isArray(typeReference)) {
    return 'array';
  }

  return null;
};

// get the array reference and destruct it in two pieces
// the sanitized type and the value to be converted to graphQL
var mapArrayType = exports.mapArrayType = function mapArrayType(fieldDefinition) {
  // array declaration is [type], so we get the first (and only) element
  var subValue = _lodash2.default.head(arrayReference(fieldDefinition));
  var subType = flattenType(subValue);

  return { subType: subType, subValue: subValue };
};